Set Warnings "-notation-overridden,-parsing".
From Coq Require Export String.
From LF Require Import Maps.

Parameter MISSING: Type.

Module Check.

Ltac check_type A B :=
    match type of A with
    | context[MISSING] => idtac "Missing:" A
    | ?T => first [unify T B; idtac "Type: ok" | idtac "Type: wrong - should be (" B ")"]
    end.

Ltac print_manual_grade A :=
    match eval compute in A with
    | Some (_ ?S ?C) =>
        idtac "Score:"  S;
        match eval compute in C with
          | ""%string => idtac "Comment: None"
          | _ => idtac "Comment:" C
        end
    | None =>
        idtac "Score: Ungraded";
        idtac "Comment: None"
    end.

End Check.

From LF Require Import Maps.
Import Check.

Goal True.

idtac "-------------------  t_apply_empty  --------------------".
idtac " ".

idtac "#> t_apply_empty".
idtac "Possible points: 1".
check_type @t_apply_empty ((forall (A : Type) (x : string) (v : A), (_ !-> v) x = v)).
idtac "Assumptions:".
Abort.
Print Assumptions t_apply_empty.
Goal True.
idtac " ".

idtac "-------------------  t_update_eq  --------------------".
idtac " ".

idtac "#> t_update_eq".
idtac "Possible points: 2".
check_type @t_update_eq (
(forall (A : Type) (m : total_map A) (x : string) (v : A), (x !-> v; m) x = v)).
idtac "Assumptions:".
Abort.
Print Assumptions t_update_eq.
Goal True.
idtac " ".

idtac "-------------------  t_update_neq  --------------------".
idtac " ".

idtac "#> t_update_neq".
idtac "Possible points: 2".
check_type @t_update_neq (
(forall (A : Type) (m : total_map A) (x1 x2 : string) (v : A),
 x1 <> x2 -> (x1 !-> v; m) x2 = m x2)).
idtac "Assumptions:".
Abort.
Print Assumptions t_update_neq.
Goal True.
idtac " ".

idtac "-------------------  t_update_shadow  --------------------".
idtac " ".

idtac "#> t_update_shadow".
idtac "Possible points: 2".
check_type @t_update_shadow (
(forall (A : Type) (m : total_map A) (x : string) (v1 v2 : A),
 (x !-> v2; x !-> v1; m) = (x !-> v2; m))).
idtac "Assumptions:".
Abort.
Print Assumptions t_update_shadow.
Goal True.
idtac " ".

idtac "-------------------  t_update_same  --------------------".
idtac " ".

idtac "#> t_update_same".
idtac "Possible points: 2".
check_type @t_update_same (
(forall (A : Type) (m : total_map A) (x : string), (x !-> m x; m) = m)).
idtac "Assumptions:".
Abort.
Print Assumptions t_update_same.
Goal True.
idtac " ".

idtac "-------------------  t_update_permute  --------------------".
idtac " ".

idtac "#> t_update_permute".
idtac "Possible points: 3".
check_type @t_update_permute (
(forall (A : Type) (m : total_map A) (v1 v2 : A) (x1 x2 : string),
 x2 <> x1 -> (x1 !-> v1; x2 !-> v2; m) = (x2 !-> v2; x1 !-> v1; m))).
idtac "Assumptions:".
Abort.
Print Assumptions t_update_permute.
Goal True.
idtac " ".

idtac " ".

idtac "Max points - standard: 12".
idtac "Max points - advanced: 12".
idtac "".
idtac "Allowed Axioms:".
idtac "functional_extensionality".
idtac "FunctionalExtensionality.functional_extensionality_dep".
idtac "plus_le".
idtac "le_trans".
idtac "le_plus_l".
idtac "add_le_cases".
idtac "Sn_le_Sm__n_le_m".
idtac "O_le_n".
idtac "".
idtac "".
idtac "********** Summary **********".
idtac "".
idtac "Below is a summary of the automatically graded exercises that are incomplete.".
idtac "".
idtac "The output for each exercise can be any of the following:".
idtac "  - 'Closed under the global context', if it is complete".
idtac "  - 'MANUAL', if it is manually graded".
idtac "  - A list of pending axioms, containing unproven assumptions. In this case".
idtac "    the exercise is considered complete, if the axioms are all allowed.".
idtac "".
idtac "********** Standard **********".
idtac "---------- t_apply_empty ---------".
Print Assumptions t_apply_empty.
idtac "---------- t_update_eq ---------".
Print Assumptions t_update_eq.
idtac "---------- t_update_neq ---------".
Print Assumptions t_update_neq.
idtac "---------- t_update_shadow ---------".
Print Assumptions t_update_shadow.
idtac "---------- t_update_same ---------".
Print Assumptions t_update_same.
idtac "---------- t_update_permute ---------".
Print Assumptions t_update_permute.
idtac "".
idtac "********** Advanced **********".
Abort.

(* 2023-09-04 16:09 *)

(* 2023-09-04 16:09 *)
